﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Net;
using System.Net.Sockets;

namespace VZ20X_WriteTime_Sample
{
    class Program
    {
        static TcpClient tcp;
		/*UNIX time*/
        private static DateTime UNIX_EPOCH = new DateTime(1970, 1, 1, 0, 0, 0, 0);

        static void Main(string[] args)
        {
            /*VZ20X IP address*/
            string VZ20X_IP = "192.168.1.1";
            /*VZ20X communication port number*/
            int port = 502;

            /*Initialize a new instance of the class*/
            tcp = new TcpClient(VZ20X_IP, port);

            /*Get current PC time*/
            DateTime targetTime = DateTime.Now;

            /*Calculate the elapsed time from 1970.1.1 00: 00: 00.000*/
            ulong unixTime = GetUnixTime(targetTime);
			
			/*Create Modbus command*/
            string w_str = "01650000000408" + unixTime.ToString("x16");
            /*Create Modbus/TCP command*/
			int str_len = w_str.Length/2;
            w_str = "00000000" + str_len.ToString("x4") + w_str;
            
			/*Send and recive*/
			string rdv_mes = SendMes(w_str);
        }

        /*************************************
       * Calculate the elapsed time from 1970.1.1 00: 00: 00.000
       * argument 　　:DateTime targetTime
       * return value :ulong
       * ************************************/
        private static ulong GetUnixTime(DateTime targetTime)
        {

            TimeSpan elapsedTime = targetTime - UNIX_EPOCH;

            return (ulong)elapsedTime.TotalMilliseconds;
        }

        /*************************************
        * Send and recive Modbus/TCP command
        * argument 　　:string mes 
        * return value :string
        * ************************************/
        static string SendMes(string mes)
        {
            NetworkStream ns = tcp.GetStream();
            ns.ReadTimeout = 1000;
            ns.WriteTimeout = 1000;

            /*hex string to byte arry*/
            byte[] SendBytes = HexStringToByte(mes);

            /*send byte arry*/
            ns.Write(SendBytes, 0, SendBytes.Length);

            System.IO.MemoryStream ms = new System.IO.MemoryStream();
            byte[] ResBytes = new byte[256];
            int resSize = 0;
            /*Recive response*/
            do
            {
                resSize = ns.Read(ResBytes, 0, ResBytes.Length);

                if (resSize == 0)
                {
                    Console.WriteLine("Sever disconnected.");
                    break;
                }

                ms.Write(ResBytes, 0, resSize);

            } while (ns.DataAvailable);
            ms.Close();

            /*byte arry to hex string*/
            string rcv_ms = ByteToHexString(ms.ToArray());
            return rcv_ms;
        }

        /*************************************
        * hex string to byte array
        * argument 　　:string s
        * return value :byte arry
        * ************************************/
        static byte[] HexStringToByte(string s)
        {
            byte[] buffer = new byte[s.Length / 2];

            for (int i = 0; i < s.Length; i += 2)
            {
                buffer[i / 2] = (byte)Convert.ToByte(s.Substring(i, 2), 16);
            }
            return buffer;
        }

        /*************************************
        * byte array to hex string 
        * argument 　　:byte[] data
        * return value　:string
        * ************************************/
        static string ByteToHexString(byte[] data)
        {
            StringBuilder sb = new StringBuilder(data.Length * 3);
            foreach (byte b in data)
            {
                sb.Append(Convert.ToString(b, 16).PadLeft(2, '0'));
            }
            return sb.ToString().ToUpper();
        }
    }
}
