import socket
import struct
import time
import codecs
import sys
import datetime 

#VZ20X IP address
VZ20X_IP = '192.168.1.2'
TARGET_PORT = 502 
BUFFER_SIZE = 512

sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
sock.connect((VZ20X_IP, TARGET_PORT))
sock.settimeout(3.0)

#MBAP Header
transactionId = 512
protocolId = 0
len = 6
unitId = 0x01

try:
	print("Send modbus request ...")
	
	#Make Modbus command to get the mesured values of 1 ～ 8CH.
	functionCode_control = 0x03 
	regi = 0x0069
	data_len = 16
	req = struct.pack('>3H 2B 2H', int(transactionId), int(protocolId), int(len), int(unitId), int(functionCode_control), int(regi), int(data_len))
	
	#Send a request to get measured values.
	sock.send(req)
	#Display receive response and received value.
	res = sock.recv(BUFFER_SIZE)
	s = struct.Struct('>3H 3B 8i')
	unpacked_data = s.unpack(res)
	print ('Values:',unpacked_data)
	
	#Make Modbus command to get the decimal point position of 1 ～ 8CH.
	functionCode_control = 0x03 
	regi = 0x012C
	data_len = 8
	sdp_req = struct.pack('>3H 2B 2H', int(transactionId), int(protocolId), int(len), int(unitId), int(functionCode_control), int(regi), int(data_len))
	
	#Send a request to get the decimal point position.
	sock.send(sdp_req)
	#Display receive response and received value.
	sdp_res = sock.recv(BUFFER_SIZE)
	s = struct.Struct('>3H 3B 8H')
	sdp_unpacked_data = s.unpack(sdp_res)
	print ('SDP:',sdp_unpacked_data)
    
	#Create a value with a decimal point position.
	#Stores measured values.
	ch1_val = unpacked_data[6]
	ch2_val = unpacked_data[7]
	ch3_val = unpacked_data[8]
	ch4_val = unpacked_data[9]
	ch5_val = unpacked_data[10]
	ch6_val = unpacked_data[11]
	ch7_val = unpacked_data[12]
	ch8_val = unpacked_data[13]
    
	#Add a decimal point to the measured value.
	for num in range(sdp_unpacked_data[6]):
		ch1_val = ch1_val/10

	for num in range(sdp_unpacked_data[7]):
		ch2_val = ch2_val/10

	for num in range(sdp_unpacked_data[8]):
		ch3_val = ch3_val/10

	for num in range(sdp_unpacked_data[9]):
		ch4_val = ch4_val/10

	for num in range(sdp_unpacked_data[10]):
		ch5_val = ch5_val/10

	for num in range(sdp_unpacked_data[11]):
		ch6_val = ch6_val/10

	for num in range(sdp_unpacked_data[12]):
		ch7_val = ch7_val/10

	for num in range(sdp_unpacked_data[13]):
		ch8_val = ch8_val/10

	#Display of measurements with a decimal point.
	print ('CH1_Values:',round(ch1_val,sdp_unpacked_data[6]))
	print ('CH2_Values:',round(ch2_val,sdp_unpacked_data[7]))
	print ('CH3_Values:',round(ch3_val,sdp_unpacked_data[8]))
	print ('CH4_Values:',round(ch4_val,sdp_unpacked_data[9]))
	print ('CH5_Values:',round(ch5_val,sdp_unpacked_data[10]))
	print ('CH6_Values:',round(ch6_val,sdp_unpacked_data[11]))
	print ('CH7_Values:',round(ch7_val,sdp_unpacked_data[12]))
	print ('CH8_Values:',round(ch8_val,sdp_unpacked_data[13]))


	time.sleep(1)
finally:
    sock.close()